package com.softslate.commerce.businessobjects.core;

import java.util.Properties;

import com.google.common.eventbus.EventBus;
import com.google.inject.Injector;
import com.softslate.commerce.daos.core.DAOFactory;

/**
 * A parent interface for all of the other interfaces handling business
 * processing. (Interfaces handling business processing are all named with the
 * suffix "Processor".)
 * <p>
 * The methods provided here allow implementations to communicate with other
 * elements of the application. In particular, retrieve general settings from
 * the database (using {@link #getSettings()}), retrive application settings
 * from the <code>/WEB-INF/classes/appSettings.properties</code> file (using
 * {@link #getAppSettings()}, retrive application components from the
 * <code>/WEB-INF/classes/appComponents.properties</code> file (using
 * {@link #getAppComponents()}, retrieve user information (using
 * {@link #getUser()}), create data access objects (using
 * {@link #getDaoFactory()}) and create other business objects (using
 * {@link #getBusinessObjectFactory()}).
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.core.BaseBusinessProcessor
 */
public interface BusinessProcessor extends BusinessObject {

	/**
	 * Retrieves the <code>Settings</code> instance previously set for this
	 * business object. The <code>Settings</code> object holds general
	 * settings stored in the <code>sscSetting</code> database table.
	 * 
	 * @return A <code>Settings</code> object that represents various settings
	 *         for the system.
	 */
	abstract Settings getSettings();

	/**
	 * Sets this business object's <code>Settings</code> instance. Used by the
	 * application's <code>BusinessObjectFactory</code> (through
	 * <code>initialize(Map parameters)</code>), to provide the system's
	 * <code>Settings</code> to this business object.
	 * 
	 * @param settings
	 *            The current <code>Settings</code> for the system, containing
	 *            various settings to be used by this business object.
	 */
	abstract void setSettings(Settings settings);

	/**
	 * Retrieves the application settings previously set for this object. The
	 * application settings are settings for information unique to a given
	 * installation, such as the location of files on the server, as opposed to
	 * the <code>Settings</code> interface, which stores settings drawn from
	 * the database. They are stored in the
	 * <code>/WEB-INF/classes/appSettings.properties</code> file under each
	 * installation.
	 * 
	 * @return A <code>Properties</code> object that represents various
	 *         settings for the application.
	 */
	public Properties getAppSettings();

	/**
	 * Sets this object's application settings. The application settings are
	 * settings for information unique to a given installation, such as the
	 * location of files on the server, as opposed to the <code>Settings</code>
	 * interface, which stores settings drawn from the database.
	 * 
	 * @param appSettings
	 *            The current application settings in effect for the system.
	 */
	public void setAppSettings(Properties appSettings);

	/**
	 * Retrieves this object's application components. The components identify
	 * which Java classes implement the various interfaces used by the system.
	 * They are stored in the
	 * <code>/WEB-INF/classes/appComponents.properties</code> file under each
	 * installation.
	 * 
	 * @return A <code>Properties</code> object that represents component
	 *         settings for the application.
	 */
	public Properties getAppComponents();

	/**
	 * Sets this object's application components. The components identify which
	 * Java classes implement the various interfaces used by the system.
	 * 
	 * @param appComponents
	 *            The current application components in effect for the system.
	 */
	public void setAppComponents(Properties appComponents);

	/**
	 * Retrieves the <code>User</code> previously set for this business
	 * object. In the struts layer, this property corresponds to the "user"
	 * attribute of each session.
	 * 
	 * @return A <code>User</code> object that represents the user currently
	 *         accessing the system.
	 */
	abstract User getUser();

	/**
	 * Sets this business object's <code>User</code>. Used by the
	 * application's <code>BusinessObjectFactory</code> (through
	 * <code>initialize(Map parameters)</code>), to provide the current
	 * <code>User</code> accessing the system to this business object. In the
	 * struts layer, this property corresponds to the "user" attribute of each
	 * session.
	 * 
	 * @param user
	 *            The current <code>User</code> accessing the system, for
	 *            which business processing is to be performed by this business
	 *            object.
	 */
	abstract void setUser(User user);

	/**
	 * Retrieves the <code>DAOFactory</code> previously set for this business
	 * object.
	 * 
	 * @return A <code>DAOFactory</code> object that can be used to create
	 *         data access objects.
	 */
	abstract DAOFactory getDaoFactory();

	/**
	 * Sets this business object's <code>DAOFactory</code>. Used by the
	 * application's <code>BusinessObjectFactory</code> (through
	 * <code>initialize(Map parameters)</code>), to provide the current
	 * <code>DAOFactory</code> for the system to this business object.
	 * 
	 * @param daoFactory
	 *            The current <code>DAOFactory</code> for the system, which
	 *            will allow this business object to create data access objects.
	 */
	abstract void setDaoFactory(DAOFactory daoFactory);

	/**
	 * Retrieves the <code>BusinessObjectFactory</code> previously set for
	 * this business object.
	 * 
	 * @return A <code>BusinessObjectFactory</code> object that can be used to
	 *         create other business objects.
	 */
	abstract BusinessObjectFactory getBusinessObjectFactory();

	/**
	 * Sets this business object's <code>BusinessObjectFactory</code>. Used
	 * by the application's <code>BusinessObjectFactory</code> (through
	 * <code>initialize(Map parameters)</code>), to provide itself to this
	 * business object, so that it can create other business objects in turn.
	 * 
	 * @param businessObjectFactory
	 *            The current <code>BusinessObjectFactory</code> for the
	 *            system, allowing this business object to create other business
	 *            objects.
	 */
	abstract void setBusinessObjectFactory(
			BusinessObjectFactory businessObjectFactory);

	/**
	 * Retrieves the <code>Injector</code> previously set.
	 * 
	 * @return A <code>Injector</code> object that can be used to create other
	 *         objects and inject them with dependencies.
	 */
	public Injector getInjector();

	/**
	 * Sets this object's <code>Injector</code> that can be used to create other
	 * objects and inject them with dependencies.
	 * 
	 * @param injector
	 *            The current <code>Injector</code> for the system, which can be
	 *            used to create other objects and inject them with
	 *            dependencies.
	 */
	public void setInjector(Injector injector);

	/**
	 * Retrieves the <code>EventBus</code> previously set for this factory.
	 * 
	 * @return A <code>EventBus</code> object that can be used to post events
	 *         and trigger subscribers to run their code.
	 */
	public EventBus getEventBus();

	/**
	 * Sets this factory's <code>EventBus</code>. This factory will pass the
	 * <code>EventBus</code> to every business object it creates.
	 * 
	 * @param eventBus
	 *            The current <code>EventBus</code> for the system, which can be
	 *            used to post events to subscribers
	 */
	public void setEventBus(EventBus eventBus);

	/**
	 * Retrieves a utility object that provides useful methods for business
	 * objects.
	 * 
	 * @return An implementation of <code>BusinessObjectUtils</code>.
	 */
	abstract BusinessObjectUtils utils();
}
