package com.softslate.commerce.businessobjects.order;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Interface for business logic methods having to do with a user's cart.
 * <p>
 * The Struts layer employs an instance implementing <code>CartProcessor</code>
 * when a user adds items to his or her cart or edits items. It is also used to
 * handle processing during checkout.
 * <p>
 * <code>CartProcessor</code> is similar in intention to
 * {@link com.softslate.commerce.businessobjects.order.OrderProcessor}.
 * <code>OrderProcessor</code> contains more generic code relating to orders,
 * while <code>CartProcessor</code> contains code specific to manuipulating a
 * user's cart.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CartProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "cartProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "cartProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.BasicCartProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface CartProcessor extends BusinessProcessor {

	/**
	 * Processes a request to add items to a user's cart. If the user's cart has
	 * not been initialized, this method initializes it first by calling
	 * {@link #initializeCart()} to create a new instance of
	 * {@link com.softslate.commerce.businessobjects.order.Order} in the
	 * {@link com.softslate.commerce.businessobjects.core.User} instance for the
	 * user. It then adds an
	 * {@link com.softslate.commerce.businessobjects.order.OrderDelivery}
	 * instance under the <code>Order</code> and
	 * {@link com.softslate.commerce.businessobjects.order.OrderItem} instances
	 * under the <code>OrderDelivery</code>.
	 * <p>
	 * This method manages the parsing of incoming parameters that identify the
	 * products, quantities, and attribute values of the items to be added. It
	 * populates <code>OrderItem</code> objects with the incoming information
	 * and then hands off the processing to the
	 * {@link #processAddItems(Collection)} method.
	 * <p>
	 * The incoming <code>Map</code> must have the following keys:
	 * <ul>
	 * <li> <code>orderProductCodes</code> (a<code>String[]</code> holding
	 * the codes of the product(s) to add)</li>
	 * <li><code>orderProductQuantities</code> (a<code>String[]</code>
	 * holding the quantities of the product(s) to add)</li>
	 * <li>and <code>orderAttributes</code> (a<code>HashMap</code> holding
	 * attribute keys and values, where the attribute keys are in the form of
	 * 'productCode|attributeCode')</li>
	 * </ul>
	 * 
	 * @param parameters
	 *            A <code>Map</code> representing the items and attributes
	 *            being added to the user's cart.
	 * @return A <code>Map</code> representing the results of the processing.
	 *         The <code>Map</code> may have one or more of the following keys
	 *         related to inventory processing: "lowStockEmails",
	 *         "missingAttributes", "rejectedOrderItems", "adjustedOrderItems",
	 *         and "newOOSSKUs".
	 * @throws Exception
	 * @see com.softslate.commerce.customer.order.CartAddAction
	 */
	Map processAddItems(Map parameters) throws Exception;

	/**
	 * Given a <code>Collection</code> of initialized <code>OrderItem</code>
	 * objects, adds the items to the user's cart. Manages the processing of
	 * inventory and discounting, updates the user's cart totals, and manages
	 * synchronizing the objects with the database.
	 * 
	 * @param newOrderItems
	 *            The <code>Collection</code> containing the
	 *            <code>OrderItem</code> objects to be added to the user's
	 *            cart.
	 * @return A <code>Map</code> containing information on the results of the
	 *         processing. The <code>Map</code> may have one or more of the
	 *         following keys related to inventory processing: "lowStockEmails",
	 *         "missingAttributes", "rejectedOrderItems", "adjustedOrderItems",
	 *         and "newOOSSKUs".
	 * @throws Exception
	 */
	Map processAddItems(Collection newOrderItems) throws Exception;

	/**
	 * Given a Collection of <code>OrderItems</code> in the user's cart whose
	 * quantities have been set, updates the totals and weights of each
	 * <code>OrderItem</code> based on the (new) quantity.
	 * 
	 * @param newOrderItems
	 *            The <code>Collection</code> containing the
	 *            <code>OrderItem</code> objects whose quantities and totals
	 *            are to be set.
	 */
	void processQuantities(Collection newOrderItems) throws Exception;

	/**
	 * Processes a request to edit a given item in a user's cart. The incoming
	 * <code>Map</code> must have the same keys required by
	 * {@link #processAddItems(Map)}, plus a key named <code>orderItem</code>,
	 * which identifies the order item being edited.
	 * 
	 * @param parameters
	 *            A <code>Map</code> identifying the item being edited and
	 *            specifying the item's new properties.
	 * @return A <code>Map</code> containing information on the results of the
	 *         processing. The <code>Map</code> may have one or more of the
	 *         following keys related to inventory processing: "lowStockEmails",
	 *         "missingAttributes", "rejectedOrderItems", "adjustedOrderItems",
	 *         and "newOOSSKUs".
	 * @throws Exception
	 * @see com.softslate.commerce.customer.order.CartItemEditAction
	 */
	Map processEditItem(Map parameters) throws Exception;

	/**
	 * Removes a given OrderItem from the user's cart, updating the cart totals
	 * in the process.
	 * 
	 * @param orderItem
	 *            The <code>OrderItem</code> being removed.
	 * @throws Exception
	 */
	Map processRemoveItem(OrderItem orderItem) throws Exception;

	/**
	 * Initializes a user's cart. Creates new instances of
	 * {@link com.softslate.commerce.businessobjects.order.Order} and
	 * {@link com.softslate.commerce.businessobjects.order.OrderDelivery}
	 * representing a user's cart. Sets the current <code>User</code> object's
	 * order property with the newly created <code>Order</code>. Synchronizes
	 * the new cart with the database.
	 * 
	 * @throws Exception
	 */
	void initializeCart() throws Exception;

	/**
	 * Processes a request to add address information to a user's cart.
	 * Originates from the checkout process in the Struts layer.
	 * 
	 * @param parameters
	 *            A <code>Map</code> representing the address information of
	 *            the current user.
	 * @throws Exception
	 * @see com.softslate.commerce.customer.order.CheckoutAction#processCheckoutAddresses(BaseForm)
	 */
	Map processCheckoutAddresses(Map parameters) throws Exception;

	/**
	 * Handles processing at the end of the checkout process, after payments
	 * have been processed. Decrements inventory, marks discounts as having been
	 * used and updates the order totals.
	 * 
	 * @return A <code>Map</code> representing the results of the processing.
	 * @throws Exception
	 * @see com.softslate.commerce.customer.order.CheckoutAction#processOrderComplete(BaseForm)
	 */
	Map processOrderComplete(Map parameters) throws Exception;

	/**
	 * Marks an order as complete at the very end of the checkout process, after
	 * payments have been processed. Sets the order's completed field with the
	 * current time. Sets the user's order with a status of "Received" and
	 * assigns an order number to the order. The "Received" status is given to
	 * all orders that have been successfully completed by the user. In other
	 * words, orders whose payments succeeded.
	 * <p>
	 * The order number is an integer given to each completed order, starting at
	 * the "orderNumberFloor" database setting (whose default is 1,000).
	 * 
	 * @return A <code>Map</code> representing the results of the processing.
	 * @throws Exception
	 * @see com.softslate.commerce.customer.order.CheckoutAction#processOrderComplete(BaseForm)
	 */
	Map markOrderComplete(Map parameters) throws Exception;

	/**
	 * Given a Collection of <code>OrderItems</code> in the user's cart whose
	 * totals have been set, updates the totals and weights of each
	 * <code>OrderDelivery</code> and <code>Order</code> they are under, and
	 * applies formatting to the totals.
	 * 
	 * @param orderItems
	 *            The <code>OrderItems</code> whose totals must be added to
	 *            the order and order delivery.
	 */
	void processTotalsAndFormatting(Collection orderItems) throws Exception;

	/**
	 * Hook method which is called any time an item or items changes in the cart.
	 * @return A <code>Map</code> representing the results of the processing.
	 * @throws Exception
	 */
	Map postProcessCartChanged() throws Exception;

	/**
	 * Called when an order is completed. Sends an order confirmation email to the customer and an order notification
	 * email to store admins.
	 * 
	 * @return A <code>Map</code> representing the results of the processing.
	 * @throws Exception
	 */
	Map sendOrderCompleteEmails(Order order, Map describe) throws Exception;
}
