package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Currency;
import java.util.Iterator;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.Product;

/**
 * Object representing an order item within the system.
 * <code>OrderItemBean</code> is the default implementation of the
 * <code>OrderItem</code> interface for the application.
 * <p>
 * The database stores order items in the <code>sscOrderItem</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>OrderItem.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.order.OrderItem
 */
@Audited
@AuditTable("AudOrderItem")
public class OrderItemBean extends BaseBusinessObject implements OrderItem,
		Serializable {

	private static final long serialVersionUID = 3258129146109245233L;

	static Log log = LogFactory.getLog(OrderItemBean.class);

	private int orderItemID = 0;

	public int getOrderItemID() {
		return (this.orderItemID);
	}

	public void setOrderItemID(int orderItemID) {
		this.orderItemID = orderItemID;
	}

	private int orderDeliveryID = 0;

	public int getOrderDeliveryID() {
		return (this.orderDeliveryID);
	}

	public void setOrderDeliveryID(int orderDeliveryID) {
		this.orderDeliveryID = orderDeliveryID;
	}

	private int orderID = 0;

	public int getOrderID() {
		return (this.orderID);
	}

	public void setOrderID(int orderID) {
		this.orderID = orderID;
	}

	private Double total;

	public Double getTotal() {
		return (this.total);
	}

	public void setTotal(Double total) {
		this.total = total;
	}

	private String formattedTotal;

	public String getFormattedTotal() {
		return (this.formattedTotal);
	}

	public void setFormattedTotal(String formattedTotal) {
		this.formattedTotal = formattedTotal;
	}

	private Double weight;

	public Double getWeight() {
		return (this.weight);
	}

	public void setWeight(Double weight) {
		this.weight = weight;
	}

	private Double costTotal;

	public Double getCostTotal() {
		return costTotal;
	}

	public void setCostTotal(Double costTotal) {
		this.costTotal = costTotal;
	}

	private int quantity;

	public int getQuantity() {
		return (this.quantity);
	}

	public void setQuantity(int quantity) {
		this.quantity = quantity;
	}

	private boolean isActive;

	public boolean getIsActive() {
		return (this.isActive);
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	private int productID;

	public int getProductID() {
		return (this.productID);
	}

	public void setProductID(int productID) {
		this.productID = productID;
	}

	private Integer manufacturerID;

	public Integer getManufacturerID() {
		return (this.manufacturerID);
	}

	public void setManufacturerID(Integer manufacturerID) {
		this.manufacturerID = manufacturerID;
	}

	private String code;

	public String getCode() {
		return (this.code);
	}

	public void setCode(String code) {
		this.code = code;
	}

	private String seoCode;

	public String getSeoCode() {
		return (this.seoCode);
	}

	public void setSeoCode(String seoCode) {
		this.seoCode = seoCode;
	}

	private String name;

	public String getName() {
		return (this.name);
	}

	public void setName(String name) {
		this.name = name;
	}

	private Double unitPrice;

	public Double getUnitPrice() {
		return (this.unitPrice);
	}

	public void setUnitPrice(Double unitPrice) {
		this.unitPrice = unitPrice;
	}

	private Double unitCost;

	public Double getUnitCost() {
		return (this.unitCost);
	}

	public void setUnitCost(Double unitCost) {
		this.unitCost = unitCost;
	}

	private Double altPrice;

	public Double getAltPrice() {
		return (this.altPrice);
	}

	public void setAltPrice(Double altPrice) {
		this.altPrice = altPrice;
	}

	private Double productWeight;

	public Double getProductWeight() {
		return (this.productWeight);
	}

	public void setProductWeight(Double productWeight) {
		this.productWeight = productWeight;
	}

	private String formattedCost;

	public String getFormattedCost() {
		return (this.formattedCost);
	}

	public void setFormattedCost(String formattedCost) {
		this.formattedCost = formattedCost;
	}

	private String formattedPrice;

	public String getFormattedPrice() {
		return (this.formattedPrice);
	}

	public void setFormattedPrice(String formattedPrice) {
		this.formattedPrice = formattedPrice;
	}

	private String formattedAltPrice;

	public String getFormattedAltPrice() {
		return (this.formattedAltPrice);
	}

	public void setFormattedAltPrice(String formattedAltPrice) {
		this.formattedAltPrice = formattedAltPrice;
	}

	private boolean isTaxed;

	public boolean getIsTaxed() {
		return (this.isTaxed);
	}

	public void setIsTaxed(boolean isTaxed) {
		this.isTaxed = isTaxed;
	}

	private String image;

	public String getImage() {
		return (this.image);
	}

	public void setImage(String image) {
		this.image = image;
	}

	private String shortDescription = null;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private String extra4 = null;

	private String extra5 = null;

	private String decrementedSKUs = null;

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getExtra4() {
		return extra4;
	}

	public void setExtra4(String extra4) {
		this.extra4 = extra4;
	}

	public String getExtra5() {
		return extra5;
	}

	public void setExtra5(String extra5) {
		this.extra5 = extra5;
	}

	public String getShortDescription() {
		return shortDescription;
	}

	public void setShortDescription(String shortDescription) {
		this.shortDescription = shortDescription;
	}

	public String getDecrementedSKUs() {
		return decrementedSKUs;
	}

	public void setDecrementedSKUs(String decrementedSKUs) {
		this.decrementedSKUs = decrementedSKUs;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public void setTaxed(boolean isTaxed) {
		this.isTaxed = isTaxed;
	}

	private Collection orderItemAttributes = null;

	public Collection getOrderItemAttributes() {
		return (this.orderItemAttributes);
	}

	public void setOrderItemAttributes(Collection orderAttributes) {
		this.orderItemAttributes = orderAttributes;
	}

	private Order order;

	private OrderDelivery orderDelivery;

	private Product product;

	private Manufacturer manufacturer;


	public Order getOrder() {
		return order;
	}

	public void setOrder(Order order) {
		this.order = order;
	}

	public OrderDelivery getOrderDelivery() {
		return orderDelivery;
	}

	public void setOrderDelivery(OrderDelivery orderDelivery) {
		this.orderDelivery = orderDelivery;
	}

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public Manufacturer getManufacturer() {
		return manufacturer;
	}

	public void setManufacturer(Manufacturer manufacturer) {
		this.manufacturer = manufacturer;
	}

	private Collection orderDiscounts = null;

	@NotAudited
	public Collection getOrderDiscounts() {
		return orderDiscounts;
	}

	public void setOrderDiscounts(Collection orderDiscounts) {
		this.orderDiscounts = orderDiscounts;
	}

	private Collection orderShippingRules = null;

	@NotAudited
	public Collection getOrderShippingRules() {
		return orderShippingRules;
	}

	public void setOrderShippingRules(Collection orderShippingRules) {
		this.orderShippingRules = orderShippingRules;
	}

	private Collection discounts = null;

	@NotAudited
	public Collection getDiscounts() {
		return discounts;
	}

	public void setDiscounts(Collection discounts) {
		this.discounts = discounts;
	}
	
	public double getItemDiscountTotal() {
		double itemDiscountTotal = 0;
		if (getOrderDiscounts() != null) {
			Iterator i = getOrderDiscounts().iterator();
			while (i.hasNext()) {
				OrderDiscount od = (OrderDiscount) i.next();
				if (od.getOrderItem() != null && od.getOrderItem().getOrderItemID() == getOrderItemID()) {
					itemDiscountTotal = itemDiscountTotal + od.getTotal();
				}
			}
		}
		return itemDiscountTotal;
	}

	public double getItemUnitPrice() {
		double itemUnitPrice = 0;
		if (getTotal() != null && getQuantity() > 0) {
			itemUnitPrice = (getTotal() - getItemDiscountTotal()) / getQuantity();
		}
		return itemUnitPrice;
	}

	public void addFormatting(Locale locale, String currencyCode) {
		if (currencyCode == null) {
			currencyCode = "USD";
		}
		NumberFormat nf = NumberFormat.getCurrencyInstance(locale);
		Currency currency = Currency.getInstance(currencyCode);
		nf.setCurrency(currency);
		if (getTotal() != null) {
			setFormattedTotal(nf.format(getTotal().doubleValue()));
		}
		if (getUnitCost() != null) {
			setFormattedCost(nf.format(getUnitCost().doubleValue()));
		}
		if (getUnitPrice() != null) {
			setFormattedPrice(nf.format(getUnitPrice().doubleValue()));
		}
		if (getAltPrice() != null) {
			setFormattedAltPrice(nf.format(getAltPrice().doubleValue()));
		}
	}
}
