package com.softslate.commerce.customer.core;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.jsp.JspException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.strutsel.taglib.html.ELLinkTag;

import com.softslate.commerce.businessobjects.core.Settings;

/**
 * Extension of the Struts <code>ELLinkTag</code>, to allow for search-engine friendly URLs for product, category and
 * other pages. As a link is generated using the <code>html:link</code> tag, this class rewrites the link into a
 * search-engine friendly format, based on the settings defined in the Settings -&gt; SEO section of the Administrator.
 * <p>
 * This class is the correlary of {@link com.softslate.commerce.customer.core.SEOFilter}, which does the reverse:
 * translating search-engine friendly URLs into formats understood by the application.
 * 
 * @author David Tobey
 */
public class AppLinkTag extends ELLinkTag {

	private static final long serialVersionUID = 3961652815942329833L;

	static Log log = LogFactory.getLog(AppLinkTag.class);

	private static String regEx1 = "/([^/]*)Page.do";

	private static String regEx2 = "/Category.do(.*?)code=([^&]*)(.*)";

	private static String regEx3 = "/Product.do(.*?)code=([^&]*)(.*)";

	private static String regEx4 = "&category=([^&]*)(.*)";

	private static String regEx5 = "&firstRow=([^&]*)(.*)";

	private static String regEx6 = "/Manufacturer.do(.*?)code=([^&]*)(.*)";

	private static String regEx7 = "/Manufacturers.do(.*)";

	private static String regEx8 = "/ProductList.do(.*)";

	private static String regEx9 = "\\?firstRow=([^&]*)(.*)";

	private static String regEx10 = "/Content.do(.*?)code=([^&]*)(.*)";

	private static Pattern pattern1 = Pattern.compile(regEx1, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern2 = Pattern.compile(regEx2, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern3 = Pattern.compile(regEx3, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern4 = Pattern.compile(regEx4, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern5 = Pattern.compile(regEx5, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern6 = Pattern.compile(regEx6, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern7 = Pattern.compile(regEx7, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern8 = Pattern.compile(regEx8, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern9 = Pattern.compile(regEx9, Pattern.CASE_INSENSITIVE);

	private static Pattern pattern10 = Pattern.compile(regEx10, Pattern.CASE_INSENSITIVE);

	/**
	 * Using the settings defined in the Settings -&gt; SEO section of the Administrator, creates a URL in search-engine
	 * friendly format. (I.e. directory style format with no query string.)
	 */
	protected String calculateURL() throws JspException {
		// Get the URL that would normally be returned from parent classes.
		String url = super.calculateURL();
		// Load in the SEO settings.
		Settings settings = (Settings) pageContext.getServletContext().getAttribute("settings");
		return createSEOURL(url, settings);
	}

	public String createSEOURL(String url, Settings settings) {
		String newURL = null;
		// If settings aren't initialized, just pass through the URL.
		if (settings == null) {
			return url;
		}
		String useSEOLinks = (String) settings.getValue("useSEOLinks");
		String seoURLMarker = (String) settings.getValue("seoURLMarker");
		String seoWelcomeMarker = (String) settings.getValue("seoWelcomeMarker");
		String seoPageMarker = (String) settings.getValue("seoPageMarker");
		String seoProductMarker = (String) settings.getValue("seoProductMarker");
		String seoCategoryMarker = (String) settings.getValue("seoCategoryMarker");
		String seoManufacturerMarker = (String) settings.getValue("seoManufacturerMarker");
		String seoProductListMarker = (String) settings.getValue("seoProductListMarker");
		String seoContentMarker = (String) settings.getValue("seoContentMarker");
		String seoSuffix = (String) settings.getValue("seoSuffix");
		String seoIncludeCategoryCodes = (String) settings.getValue("seoIncludeCategoryCodes");

		// Check to see if we should be translating at all
		if (useSEOLinks == null || !useSEOLinks.equals("1")) {
			return url;
		}
		// Snip off the session id - this makes cookies required, but is better
		// for robots.
		url = url.replaceFirst(";jsessionid=([^\\?]*)", "");

		// Clean question marks and ampers ands up a bit
		url = url.replaceAll("&&", "&");
		url = url.replaceAll("\\?&amp;", "\\?");
		url = url.replaceAll("\\?&", "\\?");

		// Append a slash only if the seoURLMarker isn't blank
		if (seoURLMarker.length() > 0) {
			seoURLMarker = seoURLMarker + "/";
		}

		// Check for the Welcome page.
		if (url.indexOf("/Welcome.do") != -1) {
			// Don't append the suffix or seo marker if the welcome marker is
			// blank
			if (seoWelcomeMarker.length() > 0) {
				newURL = url.replaceFirst("/Welcome.do", "/" + seoURLMarker + seoWelcomeMarker + seoSuffix);
			} else {
				newURL = url.replaceFirst("/Welcome.do", "/");
			}
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for Page screens such as aboutPage.do, contactPage.do
		Matcher matcher1 = pattern1.matcher(url);
		if (matcher1.find()) {
			String page = matcher1.group(1);
			newURL = url.replaceFirst(regEx1, "/" + seoURLMarker + seoPageMarker + "/" + page + seoSuffix);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for category pages.
		Matcher matcher2 = pattern2.matcher(url);
		if (matcher2.find()) {
			String firstRow = "";
			String qMarkOrQueryPart1 = matcher2.group(1);
			String code = matcher2.group(2);
			String otherParams = matcher2.group(3);
			// Check for a firstRow param
			Matcher matcher5 = pattern5.matcher(otherParams);
			if (matcher5.find()) {
				firstRow = "/" + matcher5.group(1);
				otherParams = matcher5.group(2);
			}
			String marker = seoCategoryMarker;
			newURL = "/" + seoURLMarker + marker + "/" + code + firstRow + seoSuffix;
			if (!qMarkOrQueryPart1.equals("?") || (!otherParams.equals("&") && otherParams.length() > 0)) {
				if (qMarkOrQueryPart1.equals("?") && otherParams.startsWith("&")) {
					otherParams = otherParams.substring(1);
				}
				newURL = newURL + qMarkOrQueryPart1 + otherParams;
			}
			newURL = url.replaceFirst(regEx2, newURL);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for product pages.
		Matcher matcher3 = pattern3.matcher(url);
		if (matcher3.find()) {
			String categoryCode = "";
			String qMarkOrQueryPart1 = matcher3.group(1);
			String code = matcher3.group(2);
			String otherParams = matcher3.group(3);
			// Check for a category code
			Matcher matcher4 = pattern4.matcher(otherParams);
			if (matcher4.find()) {
				categoryCode = matcher4.group(1) + "/";
				otherParams = matcher4.group(2);
			}
			String marker = seoProductMarker;
			if (seoIncludeCategoryCodes == null || seoIncludeCategoryCodes.equals("1")) {
				newURL = "/" + seoURLMarker + marker + "/" + categoryCode + code + seoSuffix;
			} else {
				newURL = "/" + seoURLMarker + marker + "/" + code + seoSuffix;

			}
			if (!qMarkOrQueryPart1.equals("?") || otherParams.length() > 0) {
				newURL = newURL + qMarkOrQueryPart1 + otherParams;
			}
			newURL = url.replaceFirst(regEx3, newURL);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for the product list page.
		Matcher matcher8 = pattern8.matcher(url);
		if (matcher8.find()) {
			String firstRow = "";
			String otherParams = matcher8.group(1);
			// Check for a firstRow param
			Matcher matcher9 = pattern9.matcher(otherParams);
			if (matcher9.find()) {
				firstRow = "/" + matcher9.group(1);
				otherParams = matcher9.group(2);
				otherParams = otherParams.replaceFirst("&", "\\?");
			}
			String marker = seoProductListMarker;
			newURL = "/" + seoURLMarker + marker + firstRow + seoSuffix;
			if (otherParams != null && otherParams.length() > 0 && !otherParams.equals("?")) {
				newURL = newURL + otherParams;
			}
			newURL = url.replaceFirst(regEx8, newURL);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for manufacturer pages.
		Matcher matcher6 = pattern6.matcher(url);
		if (matcher6.find()) {
			String firstRow = "";
			String qMarkOrQueryPart1 = matcher6.group(1);
			String code = matcher6.group(2);
			String otherParams = matcher6.group(3);
			// Check for a firstRow param
			Matcher matcher5 = pattern5.matcher(otherParams);
			if (matcher5.find()) {
				firstRow = "/" + matcher5.group(1);
				otherParams = matcher5.group(2);
			}
			String marker = seoManufacturerMarker;
			newURL = "/" + seoURLMarker + marker + "/" + code + firstRow + seoSuffix;
			if (!qMarkOrQueryPart1.equals("?") || (!otherParams.equals("&") && otherParams.length() > 0)) {
				if (qMarkOrQueryPart1.equals("?") && otherParams.startsWith("&")) {
					otherParams = otherParams.substring(1);
				}
				newURL = newURL + qMarkOrQueryPart1 + otherParams;
			}
			newURL = url.replaceFirst(regEx6, newURL);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for the manufacturer list page.
		Matcher matcher7 = pattern7.matcher(url);
		if (matcher7.find()) {
			String otherParams = matcher7.group(1);
			String marker = seoManufacturerMarker;
			newURL = "/" + seoURLMarker + marker + seoSuffix + otherParams;
			newURL = url.replaceFirst(regEx7, newURL);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// Check for content pages.
		Matcher matcher10 = pattern10.matcher(url);
		if (matcher10.find()) {
			String firstRow = "";
			String qMarkOrQueryPart1 = matcher10.group(1);
			String code = matcher10.group(2);
			String otherParams = matcher10.group(3);
			// Check for a firstRow param
			Matcher matcher5 = pattern5.matcher(otherParams);
			if (matcher5.find()) {
				firstRow = "/" + matcher5.group(1);
				otherParams = matcher5.group(2);
			}
			String marker = seoContentMarker;
			newURL = "/" + seoURLMarker + marker + "/" + code + firstRow + seoSuffix;
			if (!qMarkOrQueryPart1.equals("?") || (!otherParams.equals("&") && otherParams.length() > 0)) {
				if (qMarkOrQueryPart1.equals("?") && otherParams.startsWith("&")) {
					otherParams = otherParams.substring(1);
				}
				newURL = newURL + qMarkOrQueryPart1 + otherParams;
			}
			newURL = url.replaceFirst(regEx10, newURL);
			if (log.isDebugEnabled())
				log.debug("Translating " + url + " to " + newURL);
			return newURL;
		}
		// No match, must be a post or a dynamic page. Just let through then.
		return url;
	}
}
