package com.softslate.commerce.customer.core;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Locale;
import java.util.Properties;
import java.util.UUID;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.jstl.core.Config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.config.ForwardConfig;
import org.apache.struts.tiles.TilesRequestProcessor;

import com.google.common.eventbus.EventBus;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.GuiceBaseModule;
import com.softslate.commerce.businessobjects.core.GuiceCustomModule;
import com.softslate.commerce.businessobjects.core.RequestLog;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.events.EventBusRegistrar;
import com.softslate.commerce.customer.order.OrderInitializer;
import com.softslate.commerce.customer.product.ProductInitializer;
import com.softslate.commerce.daos.core.AppRevisionListener;
import com.softslate.commerce.daos.core.DAOFactory;
import com.softslate.commerce.daos.core.HibernateDAOFactory;

/**
 * Subclass of <code>TilesRequestProcessor</code> that negotiates each
 * application request. The subclasses of this class,
 * <code>CustomerRequestProcessor</code> and
 * <code>AdministratorRequestProcessor</code>, are specified in the
 * &lt;controller&gt; tags of the <code>struts-config.xml</code> files, as the
 * request processors for the customer and adminstrator applications
 * respectively.
 * 
 * @author David Tobey
 */
public class BaseRequestProcessor extends TilesRequestProcessor {
	static Log log = LogFactory.getLog(BaseRequestProcessor.class);

	/**
	 * Path to forward the request to if the installer is not locked. This value
	 * is <code>/installer/Installer.do</code>.
	 */
	public static String installerPath = "/installer/Installer.do";

	/**
	 * Path to forward the request to if an error occurs when connecting to the
	 * database. By default this value is <code>/installer/Installer.do</code>.
	 */
	public static String errorPath = "/installer/Installer.do";

	/**
	 * Runs initializations and other necessary routines before every request.
	 * <p>
	 * Here are the steps this method takes for each request:
	 * <ol>
	 * <li>Calls <code>sscOnPreRequestBegin()</code>, which can be overridden by
	 * a subclass to provide customizations.</li>
	 * <li>If the <code>appSettings</code> or <code>appComponents</code> objects
	 * do not exist in the application scope, initializes them.</li>
	 * <li>If a
	 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
	 * does not exist in the request scope, initializes a new one.</li>
	 * <li>If a {@link com.softslate.commerce.daos.core.DAOFactory} does not
	 * exist in the application scope, initializes a new one, including
	 * initializing Hibernate.</li>
	 * <li>Assigns the application scope's <code>DAOFactory</code> to the
	 * request's <code>BusinessObjectFactory</code>.</li>
	 * <li>If a {@link com.softslate.commerce.businessobjects.core.Settings}
	 * instance does not exist in the application scope, initializes a new one
	 * in the application and request scope, and assigns it to the application's
	 * <code>DAOFactory</code> and the request's
	 * <code>BusinessObjectFactory</code>.</li>
	 * <li>If a {@link com.softslate.commerce.businessobjects.core.User}<!-- -->
	 * does not exist in the session scope, or if the user's order has just been
	 * placed or has been corrupted somehow, initializes a new one in the
	 * session scope.</li>
	 * <li>Assigns the session scope's <code>User</code> to the request's
	 * <code>BusinessObjectFactory</code>.</li>
	 * <li>If the store's <code>categoryTree</code>,
	 * <code>builtInCategories</code> or <code>statesAndCountries</code>
	 * attributes do not exist in the application scope, initializes them using
	 * {@link com.softslate.commerce.customer.product.ProductInitializer} and
	 * {@link com.softslate.commerce.customer.order.OrderInitializer}.</li>
	 * <li>Calls <code>sscOnPreRequestEnd()</code>, which can be overridden by a
	 * subclass to provide customizations.</li>
	 * </ol>
	 * 
	 * @param request
	 *            The servlet request we are processing.
	 * @param response
	 *            The servlet response we are creating.
	 */
	protected boolean processPreprocess(HttpServletRequest request, HttpServletResponse response) {
		// Call a template method for hooks.
		boolean success = sscOnPreRequestBegin(request, response);
		if (!success) {
			return false;
		}
		String newRequest = (String) request.getAttribute("newRequest");
		boolean newRequestBool = newRequest == null || newRequest.equals("true");
		if (newRequestBool) {
			request.setAttribute("newRequest", "false");
			if (log.isInfoEnabled()) {
				log.info('\n' + "--------------- New Request: " + request.getServletPath() + "---------------");
				if (request.getSession() != null) {
					log.info('\n' + "--------------- Session ID: " + request.getSession().getId() + "---------------");
				}
			}
		}
		ActionMessages errors = new ActionMessages();

		// Load the appSettings object if it does not exist
		if (servlet.getServletContext().getAttribute("appSettings") == null) {
			if (log.isInfoEnabled())
				log.info("Application settings not found. Reinitializing.");
			ConfigPlugIn configPlugIn = new ConfigPlugIn(servlet);
			configPlugIn.initializeAppSettings();
		}
		Properties appSettings = (Properties) servlet.getServletContext().getAttribute("appSettings");

		// If the installer is not locked, forward immediately to the installer
		if (appSettings == null
				|| appSettings.getProperty("lockInstaller") == null
				|| (!appSettings.getProperty("lockInstaller").equals("locked")
						&& !appSettings.getProperty("lockInstaller").equals("true") && !appSettings.getProperty(
						"lockInstaller").equals("yes"))) {
			if (log.isWarnEnabled())
				log.warn("Installer not locked! Forwarding to the installer.");
			RequestDispatcher rd = request.getRequestDispatcher(installerPath);
			try {
				rd.forward(request, response);
				return false;
			} catch (Exception se) {
				if (log.isErrorEnabled())
					log.error("Error forwarding to installer. ", se);
				return false;
			}
		}

		// Load the appComponent object if it does not exist
		if (servlet.getServletContext().getAttribute("appComponents") == null) {
			if (log.isInfoEnabled())
				log.info("Application components not found. Reinitializing.");
			ConfigPlugIn configPlugIn = new ConfigPlugIn(servlet);
			try {
				configPlugIn.initializeAppComponents();
			} catch (Exception e) {
				// Couldn't load appComponents - most likely, because
				// appComponents-custom.properties was not created manually
				// before upgrading to 3.0.
				log.error("Couldn't load appComponents: ", e);
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", e.toString()));
				request.setAttribute(Globals.ERROR_KEY, errors);
				response.setStatus(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
				RequestDispatcher rd = request.getRequestDispatcher(errorPath);
				try {
					rd.forward(request, response);
					return false;
				} catch (Exception se) {
					if (log.isErrorEnabled())
						log.error("Error forwarding to installer. ", se);
					return false;
				}
			}
		}
		Properties appComponents = (Properties) servlet.getServletContext().getAttribute("appComponents");

		// Create the DAOFactory if it does not exist.
		if (servlet.getServletContext().getAttribute("daoFactory") == null) {
			if (log.isInfoEnabled())
				log.info("DAOFactory not found. Reinitializing.");
			ConfigPlugIn configPlugIn = new ConfigPlugIn(servlet);
			configPlugIn.initializeDAOFactory();
		}
		DAOFactory daoFactory = (DAOFactory) servlet.getServletContext().getAttribute("daoFactory");

		// Call a template method for hooks.
		boolean proceed = sscOnPreRequestMiddle(request, response);
		if (!proceed) {
			return false;
		}

		// Create this request's BusinessObjectFactory if it does not exist.
		BusinessObjectFactory bof = null;
		if (request.getAttribute("businessObjectFactory") == null) {
			if (log.isDebugEnabled())
				log.debug("Initializing BusinessObjectFactory.");
			String className = appComponents.getProperty("businessObjectFactoryImplementer");
			try {
				Class classDefinition = Class.forName(className);
				bof = (BusinessObjectFactory) classDefinition.newInstance();
			} catch (Exception e) {
				log.error("Error creating BusinessObjectFactory of type " + className, e);
			}
			request.setAttribute("businessObjectFactory", bof);
		} else {
			bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		}

		// Populate the business object factory with various properties
		bof.setAppSettings(appSettings);
		bof.setAppComponents(appComponents);
		bof.setDaoFactory(daoFactory);

		// Track whether the application scope objects should be reloaded from
		// the database with the forceSettingsReloadHitCount setting in
		// appSettings
		boolean forcedRefresh = false;
		if (newRequestBool && servlet.getServletContext().getAttribute("settings") != null && appSettings != null
				&& appSettings.getProperty("forceSettingsReloadHitCount") != null
				&& !appSettings.getProperty("forceSettingsReloadHitCount").equals("0")) {

			int forceSettingsReloadHitCount = Integer.parseInt(appSettings.getProperty("forceSettingsReloadHitCount"));
			Integer reloadCount = (Integer) servlet.getServletContext().getAttribute("reloadCount");
			if (reloadCount == null) {
				reloadCount = new Integer(1);
			}
			synchronized (servlet.getServletContext()) {
				if (reloadCount.intValue() == forceSettingsReloadHitCount) {
					if (log.isInfoEnabled())
						log.info("forceSettingsReloadHitCount met: " + forceSettingsReloadHitCount
								+ " ... Nullifying application scope settings");
					forcedRefresh = true;
					servlet.getServletContext().setAttribute("settingsBean", null);
					servlet.getServletContext().setAttribute("settings", null);
					daoFactory.setSettings(null);
					servlet.getServletContext().setAttribute("categoryTree", null);
					servlet.getServletContext().setAttribute("builtInCategories", null);
					servlet.getServletContext().setAttribute("manufacturers", null);
					servlet.getServletContext().setAttribute("statesAndCountries", null);
					servlet.getServletContext().setAttribute("reloadCount", new Integer(1));
				} else {
					servlet.getServletContext().setAttribute("reloadCount", new Integer(reloadCount.intValue() + 1));
				}
			}
		}

		// If the Settings instance does not exist, reload it into the
		// application and request scopes, and into the DAOFactory.
		if (servlet.getServletContext().getAttribute("settings") == null) {
			if (log.isInfoEnabled())
				log.info("Settings instance not found. Reinitializing.");
			Settings settings = null;
			try {
				// Declare a Settings instance for this request.
				settings =  bof.createObject(Settings.class);
				// Initialize the settings
				settings.initialize();
			} catch (Exception e) {
				// Couldn't load settings--forward to the installer error page.
				// Either the application isn't installed or there's a problem
				// connecting to the db
				log.error("Couldn't load settings", e);
				errors.add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("installer.requestSettingsError", e.toString()));
				request.setAttribute(Globals.ERROR_KEY, errors);
				response.setStatus(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
				RequestDispatcher rd = request.getRequestDispatcher(errorPath);
				try {
					rd.forward(request, response);
					return false;
				} catch (Exception se) {
					if (log.isErrorEnabled())
						log.error("Error forwarding to installer. ", se);
					return false;
				}
			}
			synchronized (servlet.getServletContext()) {
				servlet.getServletContext().setAttribute("settingsBean", settings);
				servlet.getServletContext().setAttribute("settings", settings);
				daoFactory.setSettings(settings);
			}
		}

		// Make sure the daoFactory's settings are populated
		if (daoFactory.getSettings() == null) {
			synchronized (servlet.getServletContext()) {
				Settings settings = (Settings) servlet.getServletContext().getAttribute("settings");
				daoFactory.setSettings(settings);
			}
		}

		// Copy the application's Settings into the request and the
		// request's BusinessObjectFactory.
		Settings settings = (Settings) servlet.getServletContext().getAttribute("settings");
		if (request.getAttribute("settings") == null || forcedRefresh) {
			request.setAttribute("settings", settings);
		}
		if (bof.getSettings() == null || forcedRefresh) {
			bof.setSettings(settings);
		}

		// Set the response's character encoding based on the setting
		if (newRequestBool && settings != null) {
			String charEncoding = (String) settings.getValue("characterEncoding");
			if (charEncoding != null && charEncoding.length() > 0) {
				if (log.isInfoEnabled())
					log.info("Setting encoding to: " + charEncoding);
				try {
					request.setCharacterEncoding(charEncoding);
					response.setCharacterEncoding(charEncoding);
				} catch (UnsupportedEncodingException uee) {
					if (log.isErrorEnabled())
						log.error("Error setting character encoding: ", uee);

				}
			}
		}

		// Declare a User instance in the session if one does not exist.
		HttpSession session = request.getSession();
		boolean declareUser = false;
		boolean keepLoggedIn = false;
		User user = (User) session.getAttribute("user");
		if (session != null && user == null) {
			if (log.isInfoEnabled())
				log.info("User does not exist. Initializing.");
			declareUser = true;
			// Also write a new User instance if the current one has been
			// corrupted or an order has just been completed.
		} else if (session != null) {
			if (user.getOrder() != null
					&& user.getOrder().getStatus() != null
					&& (user.getOrder().getStatus().equals("Corrupted") || user.getOrder().getStatus()
							.equals("Received"))) {
				if (log.isInfoEnabled())
					log.info("Order was just received or is corrupted.");
				// Reset checkout session variables
				request.getSession().setAttribute("shippingAndTaxesProcessed", null);
				request.getSession().setAttribute("declinedLoginBeforeCheckout", null);
				request.getSession().setAttribute("validatedPaymentForm", null);
				request.getSession().setAttribute("processorName", null);
				request.getSession().setAttribute("atLeastOneOrderPlacedInSession", Boolean.TRUE);
				declareUser = true;
				// If system is not set to end session on order completion,
				// keep the customer logged in.
				String endSessionOnOrderCompletion = (String) settings.getValue("endSessionOnOrderCompletion");
				if (user.getCustomer() != null && user.getOrder().getStatus().equals("Received")
						&& endSessionOnOrderCompletion.equals("0")) {
					keepLoggedIn = true;
				}
			}
		}
		// Create and save the new User to the session.
		if (declareUser) {
			synchronized (AppUtils.getSessionLock(request.getSession())) {
				User newUser = null;
				try {
					newUser =  bof.createObject(User.class);
				} catch (Exception e) {
					log.error("Failed to initialize a User object: " + e.toString(), e);
				}
				// Set the locale based on the browser heading as stored in the
				// session by Struts
				Locale locale = (Locale) session.getAttribute(Globals.LOCALE_KEY);
				if (locale == null
						|| (locale.getDisplayName().equals("English") && request.getHeader("User-Agent") != null && request
								.getHeader("User-Agent").indexOf("Safari") != -1)) {
					locale = Locale.getDefault();
				}

				// Override the locale if a "storeLocale" setting is defined in a format like "en" or "en_US".
				if (settings != null && settings.getValue("storeLocale") != null
						&& !settings.getValue("storeLocale").equals("")) {
					try {
						String storeLocaleSetting = settings.getValue("storeLocale");

						// For backwards compatibility
						storeLocaleSetting = storeLocaleSetting.replaceAll("_", "-");
						String[] storeLocale = storeLocaleSetting.split("-");
						if (storeLocale.length == 1) {
							locale = new Locale(storeLocale[0]);
						} else if (storeLocale.length == 2) {
							locale = new Locale(storeLocale[0], storeLocale[1]);
						} else if (storeLocale.length == 3) {
							locale = new Locale(storeLocale[0], storeLocale[1], storeLocale[2]);
						}
					} catch (Exception e) {
						log.warn("Error setting store's locale (using browser heading instead): ", e);
					}
				}
				session.setAttribute(Globals.LOCALE_KEY, locale);
				Config.set(session, Config.FMT_LOCALE, locale);
				newUser.setLocale(locale);
				
				// Keep the customer logged if called for
				if (keepLoggedIn && user != null && user.getCustomer() != null) {
					newUser.setCustomer(user.getCustomer());
				}
				// In case this was an Administrator testing things, preserve
				// the Administrator session
				if (user != null && user.getAdministrator() != null) {
					newUser.setAdministrator(user.getAdministrator());
				}
				request.getSession().setAttribute("user", newUser);
			}
		}
		// Assign the User to the BusinessObjectFactory
		user = (User) session.getAttribute("user");
		bof.setUser(user);

		// Initialize EventBus and Guice
		if (newRequestBool) {
			EventBus eventBus = createEventBus(request);
			bof.setEventBus(eventBus);
			if (settings != null && settings.getValue("initializeGuice") != null
					&& settings.getValue("initializeGuice").equals("1")) {
				initializeGuice(request);
			}
			initializeEventBus(eventBus, request);
		}
		Injector injector = (Injector) servlet.getServletContext().getAttribute(Injector.class.getName());
		if (injector != null) {
			bof.setInjector(injector);
		}

		// Create a new request log entry, either in the database or a logger
		if (newRequestBool) {
			log.info(getRequestParamsDebugMessage(request));
			processAuditLogging(request);
		}

		// If the CategoryTree does not exist, reload it into the application
		// scope.
		if (servlet.getServletContext().getAttribute("categoryTree") == null) {
			if (log.isInfoEnabled())
				log.info("CategoryTree not configured. Reinitializing.");
			ProductInitializer pi = new ProductInitializer(servlet);
			pi.initializeCategoryTree(bof);
		}

		// If the built in categories do not exist, reload them into the
		// application scope.
		if (servlet.getServletContext().getAttribute("builtInCategories") == null) {
			if (log.isInfoEnabled())
				log.info("Built In Categories not configured. Reinitializing.");
			ProductInitializer pi = new ProductInitializer(servlet);
			pi.initializeBuiltInCategories(bof);
		}

		// If the manufacturers do not exist, reload them into the
		// application scope.
		if (servlet.getServletContext().getAttribute("manufacturers") == null) {
			if (log.isInfoEnabled())
				log.info("Manufacturers not configured. Reinitializing.");
			ProductInitializer pi = new ProductInitializer(servlet);
			pi.initializeManufacturers(bof);
		}

		// If the storeDiscounts do not exist, reload them into the application
		// scope.
		if (servlet.getServletContext().getAttribute("storeDiscounts") == null) {
			if (log.isInfoEnabled())
				log.info("Store discounts not configured. Reinitializing.");
			ProductInitializer pi = new ProductInitializer(servlet);
			pi.initializeStoreDiscounts(bof);
		}

		// If the StatesAndCountries bean does not exist, reload it into the
		// application scope.
		if (servlet.getServletContext().getAttribute("statesAndCountries") == null) {
			if (log.isInfoEnabled())
				log.info("StatesAndCountries not configured. Reinitializing.");
			OrderInitializer oi = new OrderInitializer(servlet);
			oi.initializeStatesAndCountries(bof);
		}

		// Set the layout for this user from a request parameter, or a setting.
		// We do this in the BaseRequestProcessor because some admin requests
		// refer to customer-side templates.
		String defaultLayout = null;
		try {
			defaultLayout = (String) settings.getValue("defaultLayout");
		} catch (Exception e) {
			log.error("Couldn't find defaultLayout");
		}
		if (request.getParameter("layout") != null) {
			String layout = request.getParameter("layout");
			synchronized (AppUtils.getSessionLock(request.getSession())) {
				request.getSession().setAttribute("currentLayout", layout);
				request.setAttribute("currentLayout", layout);
				if (log.isInfoEnabled())
					log.info("Setting user's layout to: " + layout);
			}
		}

		// Set the layout for this user to the default layout setting.
		if (request.getSession().getAttribute("currentLayout") == null) {
			synchronized (AppUtils.getSessionLock(request.getSession())) {
				request.getSession().setAttribute("currentLayout", defaultLayout);
				request.setAttribute("currentLayout", defaultLayout);
				if (log.isInfoEnabled())
					log.info("Setting user's layout to: " + defaultLayout);
			}
		}

		// Set the csrfToken - a challenge token to prevent cross-site request
		// forgeries
		if (request.getSession().getAttribute("csrfToken") == null) {
			synchronized (AppUtils.getSessionLock(request.getSession())) {
				request.getSession().setAttribute("csrfToken", UUID.randomUUID().toString());
				if (log.isInfoEnabled())
					log.info("Setting user's csrfToken.");
			}
		}

		// Call a template method for hooks.
		success = sscOnPreRequestEnd(request, response);

		return success;
	}

	public EventBus createEventBus(HttpServletRequest request) {
		EventBus eventBus = (EventBus) request.getSession().getServletContext().getAttribute(EventBus.class.getName());

		// Do not lock every request. Only lock if the EventBus is not yet created.
		if (eventBus == null) {
			synchronized (request.getSession().getServletContext()) {
				eventBus = (EventBus) request.getSession().getServletContext().getAttribute(EventBus.class.getName());

				// If this thread was waiting on the lock, check again in case another thread created it.
				if (eventBus == null) {
					eventBus = new EventBus();
					request.getSession().getServletContext().setAttribute(EventBus.class.getName(), eventBus);
					request.setAttribute("newEventBus", Boolean.TRUE);
				}
			}
		}
		return eventBus;
	}

	public void initializeEventBus(EventBus eventBus, HttpServletRequest request) {
		if (request.getAttribute("newEventBus") != null && (Boolean) request.getAttribute("newEventBus")) {
			BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
			EventBusRegistrar ebr;
			try {
				ebr =  bof.createObject(EventBusRegistrar.class);
				ebr.initializeEventBus(eventBus);
				request.setAttribute("newEventBus", Boolean.FALSE);
			} catch (Exception e) {
				log.error("Error initializing and EventBusRegistrar");
			}
		}
	}

	public Injector initializeGuice(HttpServletRequest request) {
		Injector injector = null;
		if (request.getSession().getServletContext().getAttribute(Injector.class.getName()) == null) {
			synchronized (request.getSession().getServletContext()) {
				injector = createGuiceInjector(request);
				request.getSession().getServletContext().setAttribute(Injector.class.getName(), injector);
				HibernateDAOFactory daoFactory = (HibernateDAOFactory) request.getSession().getServletContext()
						.getAttribute("daoFactory");
				if (daoFactory != null) {
					daoFactory.setInjector(injector);
				}
			}
		}
		return injector;
	}

	public Injector createGuiceInjector(HttpServletRequest request) {
		Injector injector = null;
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		synchronized (request.getSession().getServletContext()) {
			Properties appComponents = (Properties) request.getSession().getServletContext()
					.getAttribute("appComponents");
			Properties appComponentsDefaults = (Properties) request.getSession().getServletContext()
					.getAttribute("appComponentsDefaults");
			GuiceBaseModule module = new GuiceBaseModule(appComponents, appComponentsDefaults, bof.getTransferMap(),
					bof.getEventBus());
			GuiceCustomModule customModule = new GuiceCustomModule();
			injector = Guice.createInjector(module, customModule);
		}
		return injector;
	}

	/**
	 * Performs custom processing at the beginning of each call to <code>processPreProcess</code>. It is intended to be
	 * overridden by subclasses for customizations. This method should return <code>false</code> to signal
	 * <code>processPreProcess</code> to stop futher processing and return <code>false</code> itself.
	 * 
	 * @param request
	 *            The servlet request we are processing.
	 * @param response
	 *            The servlet response we are creating.
	 * 
	 * @return <code>true</code> if the processing was successful, <code>false</code> if processing in the request
	 *         should stop.
	 */
	public boolean sscOnPreRequestBegin(HttpServletRequest request, HttpServletResponse response) {
		return true;
	}

	/**
	 * Performs custom processing in the middle of each call to <code>processPreProcess</code>, directly after the
	 * <code>daoFactory</code> has been initialized. It is intended to be overridden by subclasses for customizations.
	 * This method should return <code>false</code> to signal <code>processPreProcess</code> to stop futher processing
	 * and return <code>false</code> itself.
	 * 
	 * @param request
	 *            The servlet request we are processing.
	 * @param response
	 *            The servlet response we are creating.
	 * 
	 * @return <code>true</code> if the processing was successful, <code>false</code> if processing in the request
	 *         should stop.
	 */
	public boolean sscOnPreRequestMiddle(HttpServletRequest request, HttpServletResponse response) {
		return true;
	}

	/**
	 * Performs custom processing at the end of each call to <code>processPreProcess</code>. This method should return
	 * <code>false</code> to signal <code>processPreProcess</code> to return <code>false</code> itself.
	 * 
	 * @param request
	 *            The servlet request we are processing.
	 * @param response
	 *            The servlet response we are creating.
	 * 
	 * @return <code>true</code> if the processing was successful, <code>false</code> if processing should stop.
	 */
	public boolean sscOnPreRequestEnd(HttpServletRequest request, HttpServletResponse response) {
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		if (bof != null) {
			ActionUtils actionUtils = (ActionUtils) bof.createUtilsObject("actionUtilsImplementer");
			return actionUtils.onPreRequestEnd(request, response);
		}
		return true;
	}

	/*
	 * Overrides the default <code>process</code> method to ensure all handled exceptions while processing an ActionForm
	 * or JSP are logged.
	 */
	public void process(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		try {
			super.process(request, response);
		} catch (IOException e) {
			log.error("Exception processing request (logging and rethrowing): ", e);
			throw e;
		} catch (ServletException e) {

			// BeanUtils.populate throws an exception when processing a request param like this: 0x[]=androxgh0st
			// It's better to swallow this than to bubble it up, and just log at WARN.
			// Note: Tomcat 9 catches these errors before it hits here.
			if (e.getCause() != null && e.getCause().getClass().equals(IllegalArgumentException.class)) {
				log.warn("Exception processing request (logging and swallowing): ", e);
			} else {
				log.error("Exception processing request (logging and rethrowing): ", e);
				throw e;
			}
		}
	}

	/*
	 * Overrides the default <code>processException</code> method to ensure all handled exceptions are logged in the
	 * application log file. There should be no unhandled exceptions originating in a Struts action, unless the
	 * struts-config.xml file is changed to eliminate the global exception configuration for
	 * <code>java.lang.Exception</code>. (It is also possible for an exception to be thrown to the servlet container in
	 * this method, if one happens in the course of the exception handling itself.)
	 */
	protected ActionForward processException(HttpServletRequest request, HttpServletResponse response,
			Exception exception, ActionForm form, ActionMapping mapping) throws IOException, ServletException {
		// Warn BEFORE super.processException, which seems to trim the stack
		// trace.
		if (log.isWarnEnabled()) {
			log.warn("Exception follows: ", exception);
		}
		ActionForward result = super.processException(request, response, exception, form, mapping);

		// An exception has occurred in the Struts action. Set the response's code appropriately. For some unknown
		// reason, this status is only set for GET requests, not POSTs. Just be aware.
		response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
		return result;
	}

	/*
	 * Recursively loops through a given exception and its causes, and returns a string containing all of the chained
	 * exceptions' stack traces. Used for logging exceptions.
	 */
	public String getExceptionAsString(Throwable exception) {
		StringBuffer sb = new StringBuffer();
		sb.append(exception.toString());
		sb.append('\n');
		if (log.isInfoEnabled()) {
			StackTraceElement[] steArray = exception.getStackTrace();
			for (int i = 0; i < steArray.length; i++) {
				sb.append(steArray[i].toString());
				sb.append('\n');
			}
		}
		if (exception.getCause() != null) {
			sb.append("Cause:");
			sb.append('\n');
			sb.append(getExceptionAsString(exception.getCause()));
		}
		return sb.toString();
	}

	/*
	 * If this is an API request, forward to apiResponse.jsp.
	 */
	protected void processForwardConfig(HttpServletRequest request, HttpServletResponse response, ForwardConfig forward)
			throws IOException, ServletException {
		ForwardConfig forwardToUse = forward;
		if (Boolean.TRUE.equals(request.getAttribute("isApiRequest"))) {
			String apiForwardPath = "/WEB-INF/templates/administrator/default/json/apiResponse.jsp";
			forwardToUse = new ActionForward("/APIResponse", apiForwardPath, false, "/");
		}
		super.processForwardConfig(request, response, forwardToUse);
	}

	/*
	 * If this is an API request, forward to apiResponse.jsp.
	 */
	protected void internalModuleRelativeForward(String uri, HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		if (Boolean.TRUE.equals(request.getAttribute("isApiRequest"))) {
			String apiForwardPath = "/WEB-INF/templates/administrator/default/json/apiResponse.jsp";
			super.processForwardConfig(request, response, new ActionForward("/APIResponse", apiForwardPath, false, "/"));
		}
		super.internalModuleRelativeForward(uri, request, response);
	}

	/**
	 * Overrides the regular forward to use the user's custom layout. Replaces pages matching the system layout
	 * directory with the user's custom layout directory. If the resulting page doesn't map to an existing file, uses
	 * the original instead.
	 * <p>
	 * This method affects the forwards to the Tiles definitions for leftLayout, centerLayout, and layout. The same
	 * process happens in {@link com.softslate.commerce.customer.core.AppInsertTag} to handle forwards coming from tiles
	 * insert tags.
	 * 
	 * @param uri
	 *            The URI we're forwarding the request to, which may be modified in this method.
	 * 
	 * @param request
	 *            The servlet request we are processing.
	 * @param response
	 *            The servlet response we are creating.
	 */
	protected void doForward(String uri, HttpServletRequest request, HttpServletResponse response) throws IOException,
			ServletException {

		// Alter the URI according the user's layout.
		if (log.isDebugEnabled())
			log.debug("Setting the layout for uri: " + uri);
		String currentlayout = "currentLayout";
		if (uri.indexOf("/administrator/") > 0 || request.getSession().getAttribute(currentlayout) == null) {
			currentlayout = "adminCurrentLayout";
		}
		String layout = (String) request.getSession().getAttribute(currentlayout);
		uri = AppUtils.alterURI(uri, layout, servlet.getServletContext());
		request.setAttribute("sscIncludingPage", uri);
		super.doForward(uri, request, response);
	}

	/**
	 * Given a request object, converts all the incoming parameters to a string
	 * for debugging. The values of certain parameters such as passwords are
	 * masked for security.
	 * 
	 * @param request
	 *            The current request object.
	 * @return A String representing the name value pairs of the request
	 *         parameters.
	 */
	public String getRequestParamsDebugMessage(HttpServletRequest request) {
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		if (bof != null) {
			ActionUtils actionUtils = (ActionUtils) bof.createUtilsObject("actionUtilsImplementer");
			return actionUtils.getRequestParamsDebugMessage(request);
		}
		return "";
	}

	/**
	 * Create log entries in the application's audit logs, containing all the key information about each request.
	 * 
	 * @param request
	 *            The current request object.
	 */
	public void processAuditLogging(HttpServletRequest request) {
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		if (bof != null) {
			ActionUtils actionUtils = (ActionUtils) bof.createUtilsObject("actionUtilsImplementer");
			actionUtils.processAuditLogging(request);
		}

		// Register the requestLogID (if it exists) with the Envers AppRevisionListener.
		if (request.getAttribute("requestLog") != null) {
			RequestLog rl = (RequestLog) request.getAttribute("requestLog");
			AppRevisionListener.setRequestLogForThread(rl);
		}
	}
}
