package com.softslate.commerce.customer.product;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategorySetting;
import com.softslate.commerce.businessobjects.product.LuceneSearcher;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SearchList;
import com.softslate.commerce.businessobjects.product.SearchListBean;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * Struts action class that corresponds to the category page. Used for requests
 * to "/Category.do".
 * <p>
 * The <code>execute</code> method of this class populates a category's
 * information from the database and puts it in the request and session scopes
 * for use by the view layer. Given a
 * {@link com.softslate.commerce.customer.product.CategoryForm} with its
 * <code>code</code> property populated, it creates and initializes an instance
 * of {@link com.softslate.commerce.businessobjects.product.Category}. It then
 * invokes
 * {@link com.softslate.commerce.businessobjects.product.ProductProcessor#getCategoryFromCode(Category)}
 * to load the category's information from the database.
 * <p>
 * In the default implementation, in addition to all of the category's regular
 * properties, its <code>productListCollection</code> is populated with just
 * those <code>Product</code>s falling on the current page being displayed. The
 * paging parameter for the number of items to display comes from the
 * "categoryPageSize" setting in the <code>sscSetting</code> database table.
 * Which page to display is determined by the value of the <code>firstRow</code>
 * request parameter (which is held by the incoming <code>CategoryForm</code>
 * and copied to the <code>Category</code> object being populated).
 * <p>
 * This method puts the initialized <code>Category</code> object in the request
 * and session scopes as an attribute with the name "currentCategory". A
 * <code>Map</code> representing the category's custom settings and other
 * settings is also placed in the request and session scopes as an attribute
 * named "categorySettings".
 * <p>
 * The <code>execute</code> method next loads a <code>Map</code> of
 * attribute-only SKUs and places it in the request as an attribute named
 * "attributeSKUs" so they may be displayed on the product page. (Attribute-only
 * SKUs are SKUs which are not tied to any particular product but which may
 * still be relevant to the one of the category's products.)
 * <p>
 * Finally, three other request attributes are set from the category's data.
 * Each of these variables is initialized with default values in
 * {@link com.softslate.commerce.customer.core.BaseRequestProcessor#processPreprocess(HttpServletRequest, HttpServletResponse)}
 * and altered here with category-specific data:
 * <ul>
 * <li><code>htmlHeadTitle</code>: The category's <code>name</code> is appended
 * to this value (which is initialized with the <code>storeName</code> setting
 * from the <code>sscSetting</code> database table). By default this variable is
 * used in the &lt;title&gt; tag of the html header.</li>
 * <li><code>htmlHeadKeywords</code>: This value is replaced with the category's
 * <code>keywords</code> property. By default this variable is used in the
 * &lt;meta&gt;/keywords tag of the html header.</li>
 * <li><code>htmlHeadDescription</code>: This value is replaced with the
 * category's <code>description</code> property. By default this variable is
 * used in the &lt;meta&gt;/description tag of the html header.</li>
 * </ul>
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.product.CategoryForm
 * @see com.softslate.commerce.businessobjects.product.Category
 * @see com.softslate.commerce.businessobjects.product.ProductProcessor
 */
public class CategoryAction extends BaseAction {

	static Log log = LogFactory.getLog(CategoryAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseDynaForm baseForm = (BaseDynaForm) form;
		ProductProcessor productProcessor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
				"productProcessorImplementer");

		// Load the category
		Category category = initializeCategory(baseForm);
		boolean shouldProductsBeLoadedFromDB = shouldProductsBeLoadedFromDB(baseForm);
		category = productProcessor.getCategoryFromCode(category, shouldProductsBeLoadedFromDB);
		if (category.getCategoryID() == 0) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.categoryNotFound", category.getCode()));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			if (!category.getCode().equals("_welcome")) {
				response.setStatus(HttpServletResponse.SC_NOT_FOUND);
			}
			return mapping.findForward("failure");
		}

		// Load the products via Lucene if they weren't loaded from the DB
		SearchList searchList = null;
		if (!shouldProductsBeLoadedFromDB) {
			searchList = loadProductsWithAdvancedSearch(baseForm, category, (String) request.getParameter("sortBy"));
		}

		// Store objects in the request scope for use by JSPs
		loadAttributeOnlySKUs(baseForm, request, productProcessor);
		saveObjectsInRequestScope(category, searchList, request, baseForm);
		defineMetaTags(category, request);

		// Forward to Ajax tile if it's an Ajax request (faceted search)
		if (baseForm.getMap().get("ajax") != null && baseForm.getMap().get("ajax").equals("true")) {
			if (("default-html5").equals(baseForm.html5OrXhtml())) {
				return mapping.findForward("successAjaxHTML5");
			} else {
				return mapping.findForward("successAjax");
			}
		}

		// Set the forward based on the pageLayoutStyle setting - full layout or
		// sidebar layout
		if (category.getCategorySettings() != null) {
			CategorySetting cs = (CategorySetting) category.getCategorySettings().get("pageLayoutStyle");
			if (cs != null && cs.getValue() != null && cs.getValue().equals("layout")) {
				return mapping.findForward("successFullLayout");
			}
		}

		return mapping.findForward("success");
	}

	protected Category initializeCategory(BaseDynaForm baseForm) throws Exception {
		Category category =  baseForm.getBusinessObjectFactory().createObject(Category.class);
		BeanUtils.copyProperties(category, baseForm);

		// Set the product list parameters
		String itemsPerPage = (String) baseForm.getSettings().getValue("categoryPageSize");

		// 'View all' support
		if (baseForm.getMap().get("firstRow") != null && baseForm.getMap().get("firstRow").equals("all")) {
			itemsPerPage = (String) baseForm.getSettings().getValue("maxRecordsProcessed");
			baseForm.getMap().put("firstRow", "1");
		}
		if (log.isDebugEnabled())
			log.debug("itemsPerPage: " + itemsPerPage);
		category.setItemsPerPage(Integer.parseInt(itemsPerPage));
		return category;
	}

	protected boolean shouldProductsBeLoadedFromDB(BaseDynaForm baseForm) {
		String advancedSearchCategory = (String) baseForm.getSettings().getValue("advancedSearchCategory");
		boolean forceLoadProducts = baseForm.getMap().get("forceLoadProducts") != null
				&& baseForm.getMap().get("forceLoadProducts").equals("true");
		boolean shouldProductsBeLoadedFromDB = advancedSearchCategory.equals("0") || forceLoadProducts;
		return shouldProductsBeLoadedFromDB;
	}

	protected SearchList loadProductsWithAdvancedSearch(BaseDynaForm baseForm, Category category, String sortByParameter)
			throws Exception {

		// First create the processor to do the searching
		LuceneSearcher luceneSearcher = (LuceneSearcher) baseForm.getBusinessObjectFactory().createObject(
				"luceneSearcherImplementer");

		// Create a SearchList helper bean
		SearchList searchList =  baseForm.getBusinessObjectFactory().createObject(SearchList.class);
		BeanUtils.copyProperties(searchList, baseForm);
		baseForm.getFormUtils().eliminateDuplicateFacets(searchList);

		// Set the path to the index
		String index = (String) baseForm.getSettings().getValue("luceneIndexDir");
		searchList.setIndexDir(getServlet().getServletContext().getRealPath(index));
		searchList.setItemsPerPage(category.getItemsPerPage());
		searchList.setSearchListType(SearchListBean.SEARCH_PRODUCTS_WITHIN_CATEGORY);
		searchList.setCategoryID(category.getCategoryID());
		if (sortByParameter != null) {
			searchList.setSortBy(sortByParameter);
		} else {
			searchList.setSortBy("productOrder");
		}
		searchList.setCategoryMap((Map) getServlet().getServletContext().getAttribute("categoryMap"));

		// Load the objects and place the list in the request scope
		searchList = luceneSearcher.getSearchList(searchList);
		return searchList;
	}

	protected void loadAttributeOnlySKUs(BaseDynaForm baseForm, HttpServletRequest request,
			ProductProcessor productProcessor) throws Exception {
		String trackInventory = (String) baseForm.getSettings().getValue("trackInventory");
		String applyDiscounts = (String) baseForm.getSettings().getValue("applyDiscounts");
		if ((trackInventory != null && trackInventory.equals("1"))
				|| (applyDiscounts != null && applyDiscounts.equals("1"))) {
			Map attributeSKUs = productProcessor.loadAttributeSKUs();
			if (log.isDebugEnabled())
				log.debug("Atribute-only skus: " + attributeSKUs);
			request.setAttribute("attributeSKUs", attributeSKUs);
			Collection attributeSKUCollection = new ArrayList();
			if (!attributeSKUs.isEmpty()) {
				attributeSKUCollection = productProcessor.loadAttributeSKUCollection();
			}
			request.setAttribute("attributeSKUCollection", attributeSKUCollection);
		}
	}

	protected void saveObjectsInRequestScope(Category category, SearchList searchList, HttpServletRequest request,
			BaseDynaForm baseForm) throws Exception {
		request.setAttribute("currentCategory", category);
		request.setAttribute("categoryParam", "&category=" + category.getCode());
		String linkAction = "Category.do";
		if (category.getIsContentPage()) {
			linkAction = "Content.do";
		}
		request.setAttribute("categorySettings", category.getCategorySettings());
		if (category.getCategoryTreeID() != 0) {
			synchronized (AppUtils.getSessionLock(request.getSession())) {
				request.getSession().setAttribute("currentCategory", category);
				request.getSession().setAttribute("lastVisitedCategory", category);
				request.getSession().setAttribute("categorySettings", category.getCategorySettings());
			}
		}
		request.setAttribute("linkBase", "/" + linkAction + "?code=" + category.getCode() + "&");
		if (searchList != null) {
			request.setAttribute("searchList", searchList);
			request.setAttribute("advancedSearch", "true");
			request.setAttribute("itemCount", new Integer(searchList.getItemCount()));
			request.setAttribute("itemsPerPage", new Integer(searchList.getItemsPerPage()));
			request.setAttribute("firstRow", new Integer(searchList.getFirstRow()));
			request.setAttribute("listCollection", searchList.getSearchListCollection());
			request.setAttribute("objectTypes", searchList.getObjectTypes());
			String sort = searchList.getSortBy();
			if (!"score".equals(sort)) {
				request.setAttribute("sortByParam", "&sortBy=" + java.net.URLEncoder.encode(sort, "UTF-8"));
			}
			String facetsParam = baseForm.getFormUtils().stringArrayAsURLParam("facets", searchList.getFacets());
			request.setAttribute("facetsParam", facetsParam);
		} else {
			request.setAttribute("itemCount", new Integer(category.getItemCount()));
			request.setAttribute("itemsPerPage", new Integer(category.getItemsPerPage()));
			request.setAttribute("firstRow", new Integer(category.getFirstRow()));
			request.setAttribute("listCollection", category.getProductListCollection());
		}
	}

	protected void defineMetaTags(Category category, HttpServletRequest request) {

		String pageName = determinePageName(category);
		if (pageName.length() > 0 && !category.getCode().equals("_welcome")) {
			request.setAttribute("htmlHeadTitle", request.getAttribute("htmlHeadTitle") + " - " + pageName);
		}
		if (category.getCategorySettings() != null) {
			CategorySetting cs = (CategorySetting) category.getCategorySettings().get("seoAppendToTitleTag");
			if (cs != null && cs.getValue() != null && cs.getValue().equals("0")) {
				request.setAttribute("htmlHeadTitle", pageName);
				request.setAttribute("dontAppendToTitleTag", new Boolean(true));
			}
		}
		if (category.getIsContentPage()) {
			request.setAttribute("dontAppendToTitleTag", new Boolean(true));
		}

		// Set the meta/keywords tag
		if (category.getKeywords() != null && category.getKeywords().length() > 0) {
			request.setAttribute("htmlHeadKeywords", category.getKeywords());
		}

		// Set the meta/description tag
		boolean descriptionSet = false;
		if (category.getCategorySettings() != null) {
			CategorySetting cs = (CategorySetting) category.getCategorySettings().get("seoMetaDescription");
			if (cs != null && cs.getValue() != null && cs.getValue().length() > 0) {
				request.setAttribute("htmlHeadDescription", cs.getValue());
				descriptionSet = true;
			}
		}
		if (!descriptionSet && category.getDescription() != null) {
			String desc = category.getDescription().replaceAll("\\<.*?>", "");
			if (desc.length() > 0) {
				request.setAttribute("htmlHeadDescription", desc);
			}
		}
	}

	protected String determinePageName(Category category) {
		String pageName = "";
		if (category.getName() != null) {
			String parsedName = category.getName().replaceAll("\\<.*?>", "");
			if (parsedName.length() > 0) {
				pageName = parsedName;
			}
		}
		if (category.getCategorySettings() != null) {
			CategorySetting cs = (CategorySetting) category.getCategorySettings().get("seoTitleTag");
			if (cs != null && cs.getValue() != null && cs.getValue().length() > 0) {
				pageName = cs.getValue();
			}
		}
		return pageName;
	}
}
